#include <err.h>
#include <fcntl.h>
#include <gelf.h>
#include <stdio.h>
#include <stdint.h>
#include <stdlib.h>
#include <sysexits.h>
#include <string.h>
#include <unistd.h>

int
main(int argc, char **argv)
{
        int fd, elferr;
        Elf *e;
        Elf_Scn *scn;
        Elf_Data *data;
        size_t shstrndx;

        if (argc != 2)
                errx(EX_USAGE, "usage: %s file-name", getprogname());

        if (elf_version(EV_CURRENT) == EV_NONE)
                errx(EX_SOFTWARE, "ELF library initialization failed: %s",
		    elf_errmsg(-1));

        if ((fd = open(argv[1], O_RDONLY, 0)) < 0)
                err(EX_NOINPUT, "open \%s\" failed", argv[1]);

        if ((e = elf_begin(fd, ELF_C_READ, NULL)) == NULL)
                errx(EX_SOFTWARE, "elf_begin() failed: %s.",
		    elf_errmsg(-1));

        if (elf_kind(e) != ELF_K_ELF)
                errx(EX_DATAERR, "%s is not an ELF object.", argv[1]);

        if (elf_getshstrndx(e, &shstrndx) == 0)
                errx(EX_SOFTWARE, "getshstrndx() failed: %s.",
		    elf_errmsg(-1));

        scn = elf_getscn(e, SHN_UNDEF);
	elferr = elf_errno();
	if (elferr != 0)
		errx(EX_SOFTWARE, "elf_getscn failed: %s",
		    elf_errmsg(elferr));
	if (scn == NULL) {
		printf("scn is null\n");
		exit(0);
	}

	/* Try to get data from it */
	data = elf_getdata(scn, NULL);
	elferr = elf_errno();
	if (elferr != 0)
		errx(EX_SOFTWARE, "elf_getdata failed: %s",
		    elf_errmsg(elferr));
	if (data == NULL) {
		printf("data is null\n");
		exit(0);
	}

	printf("data = %p\n", data);
	printf("d_buf\t\t=\t%p\n", data->d_buf);
	printf("d_off\t\t=\t%ju\n", data->d_off);
	printf("d_align\t\t=\t%ju\n", data->d_align);
	printf("d_size\t\t=\t%ju\n", data->d_size);
	printf("d_type\t\t=\t%d\n", data->d_type);
	printf("d_version\t=\t%d\n", data->d_version);

	exit(0);
}

